#pragma once

#include "game/classicnote.h"
#include "graphics/classicgraphicsmanager.h"
#include "core/timeline.h"

class ClassicSprite;

class ClassicTimelineGraphicsManager : public ClassicGraphicsManager
{
public:
    explicit ClassicTimelineGraphicsManager(const std::shared_ptr<kku::Timeline<ClassicNote>>& timeline,
                                            const std::shared_ptr<ClassicSpriteFactory>& factory,
                                            const kku::microsec& visibility_offset);

    virtual void display() const override;
    virtual void update(const kku::microsec& offset) override;

    virtual void display(const std::vector<ArrowElement>& elements) const override;

protected:
    using Iterator = kku::Timeline<ClassicNote>::Iterator;

    Iterator _first;
    Iterator _last;

    const std::shared_ptr<kku::Timeline<ClassicNote>> _timeline;
    kku::microsec _display_offset;

    mutable std::map<std::shared_ptr<ClassicSprite>, Type::COUNT> _sprites;

    inline bool nothingToDraw() const noexcept;
    inline bool isVisiblyClose(const Iterator& iterator, const kku::microsec& music_offset) const noexcept;

    void fetchFirstNote(const kku::microsec& offset);
    void fetchLastNote(const kku::microsec& offset);
};
